@@ -11,18 +11,18 @@ _train or _execute (or any other specified method) in the following way:
 
     normal key string -- Is copied if in signature and passed as a named
         argument.
-    
-    node_id->key -- Is extracted (i.e. removed in original message) and passed 
+
+    node_id->key -- Is extracted (i.e. removed in original message) and passed
         as a named argument. The separator '->' is also stored available
         as the constant MSG_ID_SEP. If the key is not an argument of the
         message then the whole key is simply erased.
-        
+
 The msg returned from the inner part of the method (e.g. _execute) is then used
 to update the original message (so values can be overwritten).
 
 If args without default value are missing in the message, this will result in
 the standard Python missing-arguments-exception (this is not checked by BiNode
-itself). 
+itself).
 
 
 BiNode Return Value Options:
@@ -30,19 +30,20 @@ BiNode Return Value Options:
 
  result for execute:
      x, (x, msg), (x, msg, target)
-     
+
  result for train:
     None -- terminates training
     x, (x, msg), (x, msg, target) -- Execution is continued and
         this node will be reached at a later time to terminate training.
         If the result has the form (None, msg) then the msg is dropped (so
         it is not required to 'clear' the message manually).
-     
- result for stop_training and stop_message:
-    None -- terminates the stop_message propagation
-    (msg, target) -- If no target is specified then the remaining msg is
-        dropped (terminates the propagation).
-    
+
+ result for stop_training:
+    None -- Simply terminates the training, like for a normal node.
+    x, (x, msg), (x, msg, target) -- Causes an execute like phase, which
+        terminates when the end of the flow is reached or when EXIT_TARGET
+        is given as target value (just like during a normal execute phase).
+
 
 Magic message keys:
 ===================
@@ -55,29 +56,20 @@ keywords are treated in a special way:
      message in the return value then completely replaces the original message
      (instead of only updating it). This way a node can completely control the
      message and for example remove keys.
-     
+
  'target' -- If any template method like execute finds a 'target' keyword in
      the message then this is used as the target value in the return value.
      However, if _execute then also returns a target value this overwrites the
      target value. In global_message calls 'target' has no special meaning and
      can be used like any other keyword.
-     
+
   'method' -- Specify the name of the method that should be used instead of
       the standard one (e.g. in execute the standard method is _execute).
       An underscore is automatically added in front, so to select _execute
       one would have to provide 'execute'.
       If 'inverse' is given then the inverse dimension check will be performed
       and if no target is provided it will be set to -1.
-      
-  'method' in stop_message -- To make it possible to call 'execute' and
-      'inverse' in 'stop_message' there is some magic going on if these are
-      specified as message arguments: In addition to the normal automatic
-      extraction of an 'x' key from the message the array output of the node
-      is also stored back as 'x' in the message (overwriting the previous
-      value). Additionally the target is given a default value of 1 or -1
-      (so setting the 'method' value is sufficient for normal execution
-      or inverse).
-     
+
 """
 
 import inspect
@@ -87,64 +79,66 @@ import mdp
 # separator for node_id in message keys
 MSG_ID_SEP = "->"
 
-# methods that can overwrite docs:
-if '_stop_message' not in mdp.NodeMetaclass.DOC_METHODS:
-    mdp.NodeMetaclass.DOC_METHODS.append('_stop_message')
-
 
 class BiNodeException(mdp.NodeException):
     """Exception for BiNode problems."""
     pass
 
-    
+
 class BiNode(mdp.Node):
-    """Abstract base class for all bidirectional nodes.
-    
-    This class is not non-functional, since the arguments of the inherited
-    _execute, _train and _stop_training methods are incompatible with the calls.
-    So these methods (or _get_train_sequence instead of the last two) have to
-    be overridden.
-    
+    """Abstract base class for nodes that use bimdp features.
+
+    This class itself is not non-functional.
+
+    Derived class should, if necessary, overwrite the _bi_reset method
+    (in addition to the normal mdp.Node methods). 
+
     Note hat this class can also be used as an Adapter / Mixin for normal nodes.
     This can for example be useful for nodes which require additional data
     arguments during training or execution. These can then be encapsulated in a
     messsage. Note that BiNode has to come first in the MRO to make all this
     work.
     """
-    
+
     def __init__(self, node_id=None, stop_result=None, **kwargs):
         """Initialize BiNode.
-        
+
         node_id -- None or string which identifies the node.
-        stop_result -- A (msg, target) tupple which is used as the result for
-            stop_training (but can be overwritten by any actual results).
-            If the node has multiple training phases then this must be None or
-            an iterable with one entry for each training phase.
-        
+        stop_result -- A (msg, target) tupple which is used by stop_training.
+            If _stop_training returns a result as well then is updates /
+            overwrites the stop_result, otherwise simply stop_result is
+            returned (with x set to None).
+            If the node has multiple training phases then stop_result must be
+            None or an iterable with one entry for each training phase.
+
         kwargs are forwarded via super to the next __init__ method
         in the MRO.
         """
         self._node_id = node_id
         self._stop_result = stop_result
+        self._coroutine_instances = None
         super(BiNode, self).__init__(**kwargs)
-        
+
     ### Modified template methods from mdp.Node. ###
-    
+
     def execute(self, x, msg=None):
         """Return single value y or a result tuple.
-        
+
         x can be None, then the usual checks are omitted.
 
         The possible return types are y, (y, msg), (y, msg, target)
-        The outgoing msg carries forward the incoming message content. 
-        The last entry in a result tuple must not be None. 
+        The outgoing msg carries forward the incoming message content.
+        The last entry in a result tuple must not be None.
         y can be None if the result is a tuple.
-        
+
         This template method normally calls the corresponding _execute method
         or another method as specified in the message (using the magic 'method'
-        key. 
+        key.
         """
         if msg is None:
+            if x is None:
+                err = "Both x and msg are None."
+                raise BiNodeException(err)
             return super(BiNode, self).execute(x)
         msg_id_keys = self._get_msg_id_keys(msg)
         target = self._extract_message_key("target", msg, msg_id_keys)
@@ -161,22 +155,18 @@ class BiNode(mdp.Node):
             elif method == self._inverse:
                 self._pre_inversion_checks(x)
         result = method(x, **arg_dict)
-        return self._combine_execute_result(result, msg, target)
-    
-    def is_trainable(self):
-        """Return the return value from super."""
-        return super(BiNode, self).is_trainable()
-    
+        return self._combine_result(result, msg, target)
+
     def train(self, x, msg=None):
         """Train and return None or more if the execution should continue.
-        
+
         The possible return types are None, y, (y, msg), (y, msg, target).
         The last entry in a result tuple must not be None.
         y can be None if the result is a tuple.
-        
+
         This template method normally calls the corresponding _train method
         or another method as specified in the message (using the magic 'method'
-        key. 
+        key.
 
         Note that the remaining msg and taret values are only used if _train
         (or the requested method) returns something different from None
@@ -189,9 +179,17 @@ class BiNode(mdp.Node):
             err = "The training phase has already finished."
             raise mdp.TrainingFinishedException(err)
         if msg is None:
+            if x is None:
+                err = "Both x and msg are None."
+                raise BiNodeException(err)
             # no fall-back on Node.train because we might have a return value
             self._check_input(x)
-            self._check_train_args(x)        
+            try:
+                self._check_train_args(x)
+            except TypeError:
+                err = ("%s training seems to require " % str(self) +
+                       "additional arguments, but none were given.")
+                raise BiNodeException(err)
             self._train_phase_started = True
             x = self._refcast(x)
             return self._train_seq[self._train_phase][0](x)
@@ -205,7 +203,13 @@ class BiNode(mdp.Node):
         if x is not None:
             if (not method_name) or (method_name == "train"):
                 self._check_input(x)
-                self._check_train_args(x, **arg_dict)  
+                try:
+                    self._check_train_args(x, **arg_dict)
+                except TypeError:
+                    err = ("The given additional arguments %s " %
+                                str(arg_dict.keys()) +
+                           "are not compatible with training %s." % str(self))
+                    raise BiNodeException(err)
                 self._train_phase_started = True
                 x = self._refcast(x)
             elif method == self._inverse:
@@ -213,25 +217,27 @@ class BiNode(mdp.Node):
         result = method(x, **arg_dict)
         if result is None:
             return None
-        result = self._combine_execute_result(result, msg, target)
+        result = self._combine_result(result, msg, target)
         if (isinstance(result, tuple) and len(result) == 2 and
             result[0] is None):
             # drop the remaining msg, so that no maual clearing is required
             return None
-        return result 
-    
+        return result
+
     def stop_training(self, msg=None):
-        """Stop training phase and return None or (msg, target).
-        
-        The result tuple is then used to call stop_message on the target node.
-        The outgoing msg carries forward the incoming message content.
-        
-        This template method calls a _stop_training method from self._train_seq.
-        Note that it is not possible to select other methods via the 'method'
-        message key.
-        
+        """Stop training phase and start an execute phase with a target.
+
+        The possible return types are None, y, (y, msg), (y, msg, target).
+        For None nothing more happens, the training phase ends like for a
+        standard MDP node.
+        If a return value is given then an excute phase is started.
+
+        This template method normally calls a _stop_training method from
+        self._train_seq.
+
         If a stop_result was given in __init__ then it is used but can be
-        overwritten by the _stop_training result.
+        overwritten by the returned _stop_training result or by the 
+        msg argument provided by the BiFlow.
         """
         # basic checks
         if self.is_training() and self._train_phase_started == False:
@@ -272,112 +278,46 @@ class BiNode(mdp.Node):
             msg = stored_msg
             if target is None:
                 target = stored_stop_result[1]
-        return self._combine_stop_message_result(result, msg, target)
-    
-    ### New methods for node messaging. ###
-    
-    def stop_message(self, msg=None):
-        """Receive message and return None, msg or (msg, target).
-        
-        If the return value is (msg, target) then stop_message(msg) is called
-        on the target node.
-        
-        This template method calls the _stop_message method or another method
-        specified in the method. If the specified method is specified as
-        'execute' or 'inverse' then the x/y return value is extracted and
-        stored in the message under the key 'x' and the target defaults to
-        1 or -1 (so setting the 'method' value is sufficient for normal
-        execution or inverse).
-        """
-        if not msg:
-            return self._stop_message()
-        msg_id_keys = self._get_msg_id_keys(msg)
-        target = self._extract_message_key("target", msg, msg_id_keys)
-        method_name = self._extract_message_key("method", msg, msg_id_keys)
-        method, target = self._get_method(method_name, self._stop_message,
-                                          target)
-        if method_name == "execute" and target is None:
-            # use 1 as default target for execution, note that in order to
-            # terminate the execution one therefore has to call another method,
-            # for inverse the -1 target is already provided by _get_method
-            target = 1
-        # TODO: perform dimensionality checks for execute/inverse?
-        msg, arg_dict = self._extract_method_args(method, msg, msg_id_keys)
-        result = method(**arg_dict)
-        if method_name == "execute" or method_name == "inverse":
-            ## magic to add array result back to message
-            if result is not None:
-                if isinstance(result, tuple):
-                    x = result[0]
-                    if result[1] is not None:
-                        # result[1] must be dict
-                        result[1]["x"] = x
-                        if len(result) > 2:
-                            result = result[1:]
-                        else:
-                            result = result[1]
-                    else:
-                        if len(result) > 2:
-                            result = ({"x": x}, result[2])
-                        else:
-                            result = {"x": x}
-                else:
-                    # result must be single x value
-                    result = {"x": result}
-        return self._combine_stop_message_result(result, msg, target)
-    
-    def _stop_message(self):
-        """Hook method, overwrite when needed. 
-        
-        This default implementation only raises an exception.
-        """
-        err = ("This node (%s) does not support calling stop_message." %
-               str(self))
-        raise BiNodeException(err)
-    
+        return self._combine_result(result, msg, target)
+
     ## Additional new methods. ##
-    
+
     @property
     def node_id(self):
         """Return the node id (should be string) or None."""
         return self._node_id
-        
+
     def bi_reset(self):
         """Reset the node for the next data chunck.
-        
+
+        This template method calls the _bi_reset method.
+
         This method is automatically called by BiFlow after the processing of
         a data chunk is completed (during both training and execution).
-        
+
         All temporary data should be deleted. The internal node structure can
         be reset for the next data chunk. This is especially important if this
         node is called multiple times for a single chunk and an internal state
         keeps track of the actions to be performed for each call.
         """
+        if self._coroutine_instances is not None:
+            # delete the instance attributes to unshadow the coroutine
+            # initialization methods
+            for key in self._coroutine_instances:
+                delattr(self, key)
+            self._coroutine_instances = None
+        self._bi_reset()
+
+    def _bi_reset(self):
+        """Hook method, overwrite when needed."""
         pass
-    
-    def is_bi_training(self):
-        """Return True if a node is currently in a data gathering state.
-        
-        This method should return True if the node is gathering any data which
-        is internally stored beyond the bi_reset() call. The typical example is
-        some kind of learning in execute or message calls. 
-        Changes to the node by stop_message alone do not fall into this
-        category. 
-        
-        This method is used by the parallel package to decide if a node has to
-        to be forked or if a copy is used.
-        Basically you should ask yourself if it is sufficient to perform
-        execute or message on a copy of this Node that is later discarded, or 
-        if any data gathered during the operation should be stored. 
-        """
-        return False
-    
+
     def _request_node_id(self, node_id):
         """Return the node if it matches the provided node id.
-        
+
         Otherwise the return value is None. In this default implementation
         self is returned if node_id == self._node_id.
-        
+
         Use this method instead of directly accessing self._node_id.
         This allows a node to be associated with multiple node_ids. Otherwise
         node_ids would not work for container nodes like BiFlowNode.
@@ -386,12 +326,12 @@ class BiNode(mdp.Node):
             return self
         else:
             return None
-        
+
     ### Helper methods for msg handling. ###
-    
+
     def _get_msg_id_keys(self, msg):
         """Return the id specific message keys for this node.
-        
+
         The format is [(key, fullkey),...].
         """
         msg_id_keys = []
@@ -401,7 +341,7 @@ class BiNode(mdp.Node):
                 if node_id == self._node_id:
                     msg_id_keys.append((key, fullkey))
         return msg_id_keys
-                    
+
     @staticmethod
     def _extract_message_key(key, msg, msg_id_keys):
         """Extract and return the requested key from the message.
@@ -419,15 +359,15 @@ class BiNode(mdp.Node):
                 msg_id_keys.pop(i)
                 break
         return value
-    
+
     @staticmethod
     def _extract_method_args(method, msg, msg_id_keys):
         """Extract the method arguments form the message.
-        
+
         Return the new message and a dict with the keyword arguments (the
         return of the message is done because it can be set to None).
         """
-        arg_keys = inspect.getargspec(method)[0]   
+        arg_keys = inspect.getargspec(method)[0]
         arg_dict = dict((key, msg[key]) for key in msg if key in arg_keys)
         for key, fullkey in msg_id_keys:
             if key in arg_keys:
@@ -438,14 +378,14 @@ class BiNode(mdp.Node):
             arg_dict["msg"] = msg
             msg = None
         return msg, arg_dict
-      
+
     def _get_method(self, method_name, default_method, target=None):
         """Return the method to be called and the target return value.
-        
+
         method_name -- as provided in msg (without underscore)
         default_method -- bound method object
         target -- return target value as provided in message or None
-        
+
         If the chosen method is _inverse then the default target is -1.
         """
         if not method_name:
@@ -463,45 +403,13 @@ class BiNode(mdp.Node):
                        "there is no such method." % method_name)
                 raise BiNodeException(err)
         return method, target
-    
+
     @staticmethod
-    def _combine_stop_message_result(result, msg, target):
-        """Combine the message result with the provided values.
-        
-        result -- None, msg or (msg, target)
-        
-        The values in result always have priority.
-        If not target is available then the remaining message is dropped.
-        """
-        if not result:
-            if target is not None:
-                return (msg, target)
-            else:
-                return None
-        elif not isinstance(result, tuple):
-            # result has not target, terminate stop_message propagation
-            if target is None:
-                return None
-            if msg:
-                msg.update(result)
-            else:
-                msg = result
-            return (msg, target)
-        else:
-            # result contains target value 
-            if msg:
-                if result[0]:
-                    msg.update(result[0])
-                return msg, result[1]
-            else:
-                return result
-                
-    @staticmethod
-    def _combine_execute_result(result, msg, target):
+    def _combine_result(result, msg, target):
         """Combine the execution result with the provided values.
-        
+
         result -- x, (x, msg) or (x, msg, target)
-        
+
         The values in result always has priority.
         """
         # overwrite result values if necessary and return
@@ -523,11 +431,11 @@ class BiNode(mdp.Node):
                 return result, msg
             else:
                 return result, msg, target
-    
+
     ### Overwrite Special Methods ###
-        
+
     def __repr__(self):
-        """BiNode version of the Node representation, adding the node_id.""" 
+        """BiNode version of the Node representation, adding the node_id."""
         name = type(self).__name__
         inp = "input_dim=%s" % str(self.input_dim)
         out = "output_dim=%s" % str(self.output_dim)
@@ -542,17 +450,17 @@ class BiNode(mdp.Node):
             nid = 'node_id="%s"' % node_id
         args = ', '.join((inp, out, typ, nid))
         return name + '(' + args + ')'
-    
+
     def __add__(self, other):
         """Adding binodes returns a BiFlow.
-        
+
         If a normal Node or Flow is added to a BiNode then a BiFlow is
         returned.
         Note that if a flow is added then a deep copy is used (deep
         copies of the nodes are used).
         """
-        # unfortunately the inline import are required to avoid
-        # a cyclic import (unless one add a helper function somewhere else)
+        # unfortunately the inline imports are required to avoid
+        # a cyclic import (unless one adds a helper function somewhere else)
         if isinstance(other, mdp.Node):
             import bimdp
             return bimdp.BiFlow([self, other])
@@ -564,5 +472,86 @@ class BiNode(mdp.Node):
         else:
             # can delegate old cases
             return super(BiNode, self).__add__(other)
-        
-        
\ No newline at end of file
+
+
+class PreserveDimBiNode(BiNode, mdp.PreserveDimNode):
+    """BiNode version of the PreserveDimNode."""
+    pass
+
+
+### Helper Functions / Decorators ###
+
+def binode_coroutine(args=None, defaults=()):
+    """Decorator for the convenient definition of BiNode couroutines.
+    
+    This decorator takes care of all the boilerplate code to use a coroutine
+    as a BiNode method for continuations (which is more elegant and convenient
+    than using a a state machine implementation). 
+
+    args -- List of string names of the additional arguments. Note that the
+        standard 'x' array is always given as the first value. So if n args
+        are requested the yield will return n+1 values.
+    defaults -- Tuple of default values for the arguments. If this tuple has
+        n elements, they correspond to the last n elements in 'args'
+        (following the convention of inspect.getargspec).
+    
+    Internally there are three methods/functions:
+        - The user defined function containing the original coroutine code.
+          This is only stored in the decorator closure.
+        - A new method ('_coroutine_initialization') with the name and
+          signature  of the decorated coroutine, which internally handles the
+          first initialization of the coroutine instance.
+          This method is returned by the decorator.
+        - A method with the signature specified by the 'args' for the
+          decorator.
+          After the coroutine has been initialized this
+          method shadows the initialization method in the class instance
+          (using an instance attribute to shadow the class attribute).
+    """
+    if args is None:
+        args = ["self", "x"]
+    else:
+        args = ["self", "x"] + args
+    def _binode_coroutine(coroutine):
+        # the original coroutine is only stored in this closure
+        infodict = mdp.NodeMetaclass._function_infodict(coroutine)
+        original_name = infodict["name"]
+        ## create the coroutine interface method
+        def _coroutine_interface(self, *args):
+            try:
+                return self._coroutine_instances[original_name].send(args)
+            except StopIteration, exception:
+                delattr(self, original_name)
+                del self._coroutine_instances[original_name]
+                if len(exception.args):
+                    return exception.args
+                else:
+                    return None
+        # turn the signature into the one specified by the args
+        interface_infodict = infodict.copy()
+        interface_infodict["signature"] = ", ".join(args)
+        interface_infodict["defaults"] = defaults
+        coroutine_interface = mdp.NodeMetaclass._wrap_function(
+                                    _coroutine_interface, interface_infodict)
+        ## create the initialization method
+        def _coroutine_initialization(self, *args):
+            coroutine_instance = coroutine(self, *args)
+            bound_coroutine_interface = coroutine_interface.__get__(
+                                                        self, self.__class__)
+            if self._coroutine_instances is None:
+                self._coroutine_instances = dict()
+            self._coroutine_instances[original_name] = coroutine_instance
+            setattr(self, original_name, bound_coroutine_interface)
+            try:
+                return coroutine_instance.next()
+            except StopIteration, exception:
+                delattr(self, original_name)
+                del self._coroutine_instances[original_name]
+                if len(exception.args):
+                    return exception.args
+                else:
+                    return None
+        coroutine_initialization = mdp.NodeMetaclass._wrap_function(
+                                    _coroutine_initialization, infodict)
+        return coroutine_initialization
+    return _binode_coroutine
