@@ -2,7 +2,9 @@
 from biflownode import BiFlowNode
 from bilayer import CloneBiLayerException, CloneBiLayer
 from biswitchboard import *
+from bihtmlvisitor import BiHiNetHTMLVisitor, show_biflow
 
 del biflownode
 del bilayer
 del biswitchboard
+del bihtmlvisitor
