@@ -113,10 +113,10 @@ class MDPDeprecationWarning(DeprecationWarning, MDPWarning):
 
 import configuration
 
-__version__ = '3.2'
+__version__ = '3.3'
 __revision__ = configuration.get_git_revision()
 __authors__ = 'MDP Developers'
-__copyright__ = '(c) 2003-2011 mdp-toolkit-devel@lists.sourceforge.net'
+__copyright__ = '(c) 2003-2012 mdp-toolkit-devel@lists.sourceforge.net'
 __license__ = 'BSD License, see COPYRIGHT'
 __contact__ = 'mdp-toolkit-users@lists.sourceforge.net'
 __homepage__ = 'http://mdp-toolkit.sourceforge.net'
@@ -167,14 +167,6 @@ import hinet
 import parallel
 from test import test
 
-# clean up namespace
-del signal_node
-del linear_flows
-del classifier_node
-del helper_funcs
-del configuration
-del repo_revision
-
 # explicitly set __all__, mainly needed for epydoc
 __all__ = ['config',
            'CheckpointFlow',
@@ -223,7 +215,9 @@ utils.fixup_namespace(__name__, __all__,
                       ('signal_node',
                        'linear_flows',
                        'helper_funcs',
-                       'extension',
                        'classifier_node',
                        'configuration',
-                       ))
+                       'repo_revision',
+                       'extension',
+                       ),('extension',
+                          'configuration'))
