@@ -137,15 +137,15 @@ class config(object):
         """Return nicely formatted info about MDP.
 
         >>> print mdp.config.info()                           # doctest: +SKIP
-                  python: 2.6.6.final.0
-                   scipy: 0.7.2
-            scipy signal: 0.7.2
-         parallel python: 1.6.0
-                  shogun: v0.9.3_r4889_2010-05-27_20:52_
-                  libsvm: NOT AVAILABLE: No module named svm
-              new symeig: NOT AVAILABLE: symeig version too old
-                  symeig: wrap_eigh
-                  joblib: 0.4.6
+                  python: 2.7.2.final.0
+                     mdp: 3.3, MDP-3.2-9-g4bc7356+
+         parallel python: 1.6.1-monkey-patched
+                  shogun: v1.1.0_02ce3cd_2011-12-12_08:17_
+                  libsvm: libsvm.so.3
+                  joblib: 0.5.4
+                 sklearn: 0.9
+                    numx: scipy 0.9.0
+                  symeig: scipy.linalg.eigh
 
         This function is used to provide the py.test report header and
         footer.
@@ -271,38 +271,45 @@ class _sys_stdout_replaced(object):
         sys.stdout = self.sysstdout
 
 def _pp_needs_monkeypatching():
-    # check if we are on one of those broken system were
-    # parallel python is affected by
-    # http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=620551
-    # this is a minimal example to reproduce the problem
-    # XXX IMPORTANT XXX
-    # This function only works once, i.e. at import
-    # if you attempt to call it again afterwards,
-    # it does not work [pp does not print the error twice]
+    # only run this function the first time mdp is imported
+    # otherwise reload(mdp) breaks
 
-    # we need to hijack stdout here, because pp does not raise
-    # exceptions: it writes to stdout directly!!!
+    if not hasattr(mdp, '_pp_needs_monkeypatching'):
+        # check if we are on one of those broken system were
+        # parallel python is affected by
+        # http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=620551
+        # this is a minimal example to reproduce the problem
+        # XXX IMPORTANT XXX
+        # This function only works once, i.e. at import
+        # if you attempt to call it again afterwards,
+        # it does not work [pp does not print the error twice]
 
-    # pp stuff
-    import pp
-    server = pp.Server()
-    with _sys_stdout_replaced() as capture:
-        server.submit(lambda: None, (), (), ('numpy',))()
-        server.destroy()
+        # we need to hijack stdout here, because pp does not raise
+        # exceptions: it writes to stdout directly!!!
 
-    # read error from hijacked stdout
-    error = capture.getvalue()
+        # pp stuff
+        import pp
+        server = pp.Server()
+        with _sys_stdout_replaced() as capture:
+            server.submit(lambda: None, (), (), ('numpy',))()
+            server.destroy()
 
-    return 'ImportError' in error
+        # read error from hijacked stdout
+        error = capture.getvalue()
+        mdp._pp_needs_monkeypatching = 'ImportError' in error
+
+    return mdp._pp_needs_monkeypatching
 
 def set_configuration():
     # set python version
     config.ExternalDepFound('python', '.'.join([str(x)
                                                 for x in sys.version_info]))
-    config.ExternalDepFound('mdp', mdp.__version__+', '+mdp.__revision__)
+    version = mdp.__version__
+    if mdp.__revision__:
+        version += ', ' + mdp.__revision__
+    config.ExternalDepFound('mdp', version)
 
     # parallel python dependency
-    config.pp_monkeypatch_dirname = None
     try:
         import pp
         # set pp secret if not there already
@@ -320,16 +327,27 @@ def set_configuration():
         if os.getenv('MDP_DISABLE_PARALLEL_PYTHON'):
             config.ExternalDepFailed('parallel_python', 'disabled')
         else:
-            if _pp_needs_monkeypatching():
-                if os.getenv('MDP_DISABLE_MONKEYPATCH_PP'):
-                    config.ExternalDepFailed('parallel_python', pp.version +
-                                             ' broken on Debian')
-                else:
-                    config.ExternalDepFound('parallel_python', pp.version +
-                                            '-monkey-patched')
-                    config.pp_monkeypatch_dirname = tempfile.gettempdir()
+            # even if we can import pp, starting the server may still fail
+            # for example with:
+            # OSError: [Errno 12] Cannot allocate memory
+            try:
+                server = pp.Server()
+                server.destroy()
+            except Exception, exc:
+                # no idea what exception the pp server may raise
+                # we need to catch all here...
+                config.ExternalDepFailed('parallel_python', exc)
             else:
-                config.ExternalDepFound('parallel_python', pp.version)
+                if _pp_needs_monkeypatching():
+                    if os.getenv('MDP_DISABLE_MONKEYPATCH_PP'):
+                        config.ExternalDepFailed('parallel_python', pp.version +
+                                                 ' broken on Debian')
+                    else:
+                        config.ExternalDepFound('parallel_python', pp.version +
+                                                '-monkey-patched')
+                        config.pp_monkeypatch_dirname = tempfile.gettempdir()
+                else:
+                    config.ExternalDepFound('parallel_python', pp.version)
 
     # shogun
     try:
@@ -342,19 +360,20 @@ def set_configuration():
     else:
         if os.getenv('MDP_DISABLE_SHOGUN'):
             config.ExternalDepFailed('shogun', 'disabled')
-        # From now on just support shogun >= 1.0
-        # Between 0.10 to 1.0 there are too many API changes...
-        try:
-            version = sgKernel.Version_get_version_release()
-        except AttributeError:
-            config.ExternalDepFailed('shogun',
-                                     'too old, upgrade to at least version 1.0')
         else:
-            if not version.startswith('v1.'):
+            # From now on just support shogun >= 1.0
+            # Between 0.10 to 1.0 there are too many API changes...
+            try:
+                version = sgKernel.Version_get_version_release()
+            except AttributeError:
                 config.ExternalDepFailed('shogun',
-                                         'too old, upgrade to at least version 1.0.')
+                                         'too old, upgrade to at least version 1.0')
             else:
-                config.ExternalDepFound('shogun', version)
+                if not version.startswith('v1.'):
+                    config.ExternalDepFailed('shogun',
+                                             'too old, upgrade to at least version 1.0.')
+                else:
+                    config.ExternalDepFound('shogun', version)
 
     # libsvm
     try:
@@ -396,7 +415,7 @@ def set_configuration():
         config.ExternalDepFailed('sklearn', exc)
     except AttributeError, exc:
         config.ExternalDepFailed('sklearn', exc)
-    else:   
+    else:
         if os.getenv('MDP_DISABLE_SKLEARN'):
             config.ExternalDepFailed('sklearn', 'disabled')
         elif _version_too_old(version, (0,6)):
