@@ -2,13 +2,10 @@ from graph import ( Graph, GraphEdge, GraphException, GraphNode,
                     GraphTopologicalException, is_sequence,
                     recursive_map, recursive_reduce)
 
-del graph
-
 __all__ = ['Graph', 'GraphEdge', 'GraphException', 'GraphNode',
            'GraphTopologicalException', 'is_sequence',
            'recursive_map', 'recursive_reduce']
 
 from mdp.utils import fixup_namespace
-import sys as _sys
 fixup_namespace(__name__, __all__,
-                ('graph',))
+                ('graph','fixup_namespace',))
