@@ -55,11 +55,12 @@ __all__ = ['FlowNode', 'Layer', 'SameInputLayer', 'CloneLayer',
            ]
 
 from mdp.utils import fixup_namespace
-import sys as _sys
 fixup_namespace(__name__, __all__,
                 ('flownode',
                  'layer',
                  'switchboard',
                  'hinet_Visitor',
                  'switchboard_factory',
+                 'utils',
+                 'fixup_namespace'
                  ))
