@@ -6,7 +6,6 @@ import inspect as _inspect
 import warnings as _warnings
 import traceback as _traceback
 import cPickle as _cPickle
-import warnings as _warnings
 import tempfile as _tempfile
 import copy as _copy
 
@@ -250,8 +249,8 @@ class Flow(object):
 
         if not isinstance(data_iterables, list):
             err_str = ("'data_iterables' must be either a list of "
-                       "iterables or an array, but got %s" %
-                       str(type(data_iterables)))
+                       "iterables or an array, and not %s" %
+                       type(data_iterables))
             raise FlowException(err_str)
 
         # check that all elements are iterable
