@@ -2,15 +2,12 @@
 
 """These are test functions for MDP classifiers.
 """
-import inspect
-
 from _tools import *
 
 from mdp import ClassifierNode
 from mdp.nodes import (SignumClassifier, PerceptronClassifier,
                        SimpleMarkovClassifier, DiscreteHopfieldClassifier,
                        KMeansClassifier)
-from mdp.utils import weighted_choice
 
 def _sigmoid(t):
     return 1.0 / (1.0 + numx.exp(-t))
@@ -152,7 +149,7 @@ def testDiscreteHopfieldClassifier():
         # check, if a noisy pattern is recreated
         noisy = numx.array(p)
         for i in xrange(len(noisy)):
-            if numx.random.random() > 0.9:
+            if numx.random.random() > 0.95:
                 noisy[i] = not noisy[i]
         retrieved = h.label(numx.array([noisy]))
         # Hopfield nets are blind for inversion, need to check either case
