@@ -5,7 +5,7 @@ def test_Node_deepcopy_lambda():
     should not throw an Exception"""
     generic_node = mdp.Node()
     generic_node.lambda_function = lambda: 1
-    copy_node = generic_node.copy()
+    generic_node.copy()
 
 def test_Flow_deepcopy_lambda():
     """Copying a Flow with a lambda member function
@@ -13,4 +13,4 @@ def test_Flow_deepcopy_lambda():
     generic_node = mdp.Node()
     generic_node.lambda_function = lambda: 1
     generic_flow = mdp.Flow([generic_node])
-    copy_flow = generic_flow.copy()
+    generic_flow.copy()
