@@ -45,6 +45,13 @@ def test_random_rot():
         assert_almost_equal(numx_linalg.det(x), 1., 4)
         assert_array_almost_equal(y, numx.eye(dim), 4)
 
+def test_random_rot_determinant_sign():
+    x = utils.random_rot(4)
+    assert_almost_equal(numx_linalg.det(x), 1., 4)
+    x = utils.random_rot(5)
+    assert_almost_equal(numx_linalg.det(x), 1., 4)
+
+
 def test_casting():
     x = numx_rand.random((5,3)).astype('d')
     y = 3*x
@@ -133,3 +140,13 @@ def test_QuadraticForm_non_symmetric_raises():
     H = numx_rand.random((10,10))
     py.test.raises(mdp.utils.QuadraticFormException,
                    utils.QuadraticForm, H)
+
+def test_nongeneral_svd_bug():
+    a = numx.array([[ 0.73083003,  0.        ,  0.7641788 ,  0.        ],
+                    [ 0.        ,  0.        ,  0.        ,  0.        ],
+                    [ 0.7641788 ,  0.        ,  0.79904932,  0.        ],
+                    [ 0.        ,  0.        ,  0.        ,  0.        ]])
+    w, z = utils.nongeneral_svd(a)
+    diag = numx.diagonal(utils.mult(utils.hermitian(z),
+                                    utils.mult(a, z))).real
+    assert_array_almost_equal(diag, w, 12)
